%global toolchain clang

Name:		llvm
Version:	16.0.6
Release:	2%{?dist}
Summary:	Low Level Virtual Machine, modular and reusable compiler and toolchain
License:	Apache License v2.0 with LLVM Exceptions
URL:		http://llvm.org
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}/llvm-%{version}.src.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}/cmake-%{version}.src.tar.xz
Source2:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}/third-party-%{version}.src.tar.xz

Patch0001:	0001-SystemZ-Improve-error-messages-for-unsupported-reloc.patch
Patch0002:	0001-cmake-Add-LLVM_UNITTEST_LINK_FLAGS-option.patch
Patch3001:	0001-llvm-Add-install-targets-for-gtest.patch
Patch3002:	0201-third-party-Add-install-targets-for-gtest.patch
Patch5000:	Disable-INSTANTIATE_TEST_SUITE_P-OnAarch64.patch

%define maj_ver %(echo %{version} | cut -d. -f1)
%define min_ver %(echo %{version} | cut -d. -f2)
%define patch_ver %(echo %{version} | cut -d. -f3)

BuildRequires:	gcc gcc-c++ clang cmake ninja-build
BuildRequires:	zlib-devel libffi-devel ncurses-devel binutils-devel libedit-devel
BuildRequires:	python3-sphinx python3-recommonmark
# for valgrind support
BuildRequires:	valgrind-devel
# fo patchfix.py
BuildRequires:	python3-devel

Requires:	%{name}-libs = %{version}-%{release}
Provides:	llvm(major) = %{maj_ver}

%description
The LLVM Project is a collection of modular and reusable compiler
and toolchain technologies, including llvm, clang, lldb, compiler-rt,
and so on.The LLVM Core libraries provide a modern source- and
target-independent optimizer, along with code generation support for
many popular CPUs. These libraries are built around a well specified
code representation known as the LLVM intermediate representation ("LLVM IR").

Documentation is https://llvm.org/docs/.


%package devel
Summary:	Libraries and header files for LLVM
Requires:	%{name} = %{version}-%{release}
Requires:	%{name}-libs = %{version}-%{release}
# for -ledit to the linker flags
Requires:	libedit-devel
Requires:	%{name}-test = %{version}-%{release}
Provides:	llvm-devel(major) = %{maj_ver}
Provides:	llvm-static(major) = %{maj_ver}
Provides:	llvm-static

%description devel
Library and header files to develop programs using the LLVM infrastructure.


%package libs
Summary:	LLVM shared libraries

%description libs
Shared libraries for LLVM.

%package cmake-utils
Summary: CMake utilities shared across LLVM subprojects

%description cmake-utils
CMake utilities shared across LLVM subprojects.
This is for internal use by LLVM packages only.

%package test
Summary:	LLVM regression tests
Requires:	%{name} = %{version}-%{release}
Requires:	%{name}-libs = %{version}-%{release}
Provides:	llvm-test(major) = %{maj_ver}
Provides:	llvm-googletest

%description test
LLVM regression tests adn LLVM's modified googletest sources.


%prep
%setup -T -q -b 1 -n cmake-%{version}.src
cd ..
mv cmake-%{version}.src cmake

%setup -T -q -b 2 -n third-party-%{version}.src
%patch3002 -p2
cd ..
mv third-party-%{version}.src third-party

%setup -T -q -b 0 -n llvm-%{version}.src
%patch0001 -p2
%patch0002 -p2
%patch3001 -p2
%ifarch aarch64
%patch5000 -p1
%endif

pathfix.py -i %{__python3} -p -n -k -as \
	test/BugPoint/compile-custom.ll.py \
	tools/opt-viewer/*.py \
	utils/update_cc_test_checks.py

%build
%global _lto_cflags -flto=thin

%cmake	-G Ninja \
	-DCMAKE_C_FLAGS="-gdwarf-4" \
	-DCMAKE_CXX_FLAGS="-gdwarf-4" \
	-DBUILD_SHARED_LIBS:BOOL=OFF \
	-DLLVM_PARALLEL_LINK_JOBS=1 \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DLLVM_LIBDIR_SUFFIX=64 \
	-DLLVM_TARGETS_TO_BUILD="all" \
        -DLLVM_EXPERIMENTAL_TARGETS_TO_BUILD='' \
	-DLLVM_ENABLE_LIBCXX:BOOL=OFF \
	-DLLVM_ENABLE_ZLIB:BOOL=ON \
	-DLLVM_ENABLE_FFI:BOOL=ON \
	-DLLVM_ENABLE_RTTI:BOOL=ON \
	-DLLVM_USE_PERF:BOOL=ON \
	-DLLVM_BINUTILS_INCDIR=%{_includedir} \
	-DLLVM_BUILD_RUNTIME:BOOL=ON \
	-DLLVM_INCLUDE_TOOLS:BOOL=ON \
	-DLLVM_BUILD_TOOLS:BOOL=ON \
	-DLLVM_INCLUDE_TESTS:BOOL=ON \
	-DLLVM_BUILD_TESTS:BOOL=ON \
	-DLLVM_INSTALL_GTEST:BOOL=ON \
	-DLLVM_LIT_ARGS=-v \
	-DLLVM_INCLUDE_EXAMPLES:BOOL=ON \
	-DLLVM_BUILD_EXAMPLES:BOOL=OFF \
	-DLLVM_INCLUDE_UTILS:BOOL=ON \
	-DLLVM_INSTALL_UTILS:BOOL=ON \
	-DLLVM_UTILS_INSTALL_DIR:PATH=%{_bindir} \
	-DLLVM_TOOLS_INSTALL_DIR:PATH=bin \
	-DLLVM_INCLUDE_DOCS:BOOL=ON \
	-DLLVM_BUILD_DOCS:BOOL=ON \
	-DLLVM_ENABLE_SPHINX:BOOL=ON \
	-DLLVM_ENABLE_DOXYGEN:BOOL=OFF \
	-DLLVM_VERSION_SUFFIX='' \
	-DLLVM_BUILD_LLVM_DYLIB:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_BUILD_EXTERNAL_COMPILER_RT:BOOL=ON \
	-DLLVM_INSTALL_TOOLCHAIN_ONLY:BOOL=OFF \
	-DLLVM_DEFAULT_TARGET_TRIPLE=%{_host} \
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
	-DCMAKE_INSTALL_PREFIX=/usr \
	-DLLVM_INSTALL_SPHINX_HTML_DIR=%{_pkgdocdir}/html \
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3 \
	-DLLVM_INCLUDE_BENCHMARKS=OFF \
	-DLLVM_UNITTEST_LINK_FLAGS="-Wl,-plugin-opt=O0"

# Build libLLVM.so first to help reduce OOM errors during concurrent building.
%cmake_build --target LLVM
%cmake_build

%install
%cmake_install
rm -rf %{buildroot}/%{_pkgdocdir}/html

# for lit tests
mkdir -p %{buildroot}/%{_bindir}
install -m 0755 %{_vpath_builddir}/bin/llvm-isel-fuzzer %{buildroot}%{_bindir}
install -m 0755 %{_vpath_builddir}/bin/llvm-opt-fuzzer %{buildroot}%{_bindir}

rm -rf test/tools/UpdateTestChecks
install %{_vpath_builddir}/lib64/libLLVMTestingSupport.a %{buildroot}%{_libdir}
install %{_vpath_builddir}/lib64/libLLVMTestingAnnotations.a %{buildroot}%{_libdir}

# Add symlink to lto plugin in the binutils plugin directory
mkdir -p %{buildroot}%{_libdir}/bfd-plugins/
ln -s -t %{buildroot}%{_libdir}/bfd-plugins/ ../LLVMgold.so

(cd %{buildroot}/%{_bindir} ; ln -s llvm-config llvm-config-%{maj_ver})

cp -Rv ../cmake/Modules/* %{buildroot}%{_libdir}/cmake/llvm

mkdir -p %{buildroot}%{_datadir}/llvm/cmake
cp -Rv ../cmake/* %{buildroot}%{_datadir}/llvm/cmake

%check

LD_LIBRARY_PATH=%{buildroot}/%{_libdir}  %{__ninja} check-all -C %{_vpath_builddir}


%files
%license LICENSE.TXT
%{_bindir}/*
%exclude %{_bindir}/llvm-config
%exclude %{_bindir}/llvm-config-%{maj_ver}
%exclude %{_bindir}/not
%exclude %{_bindir}/count
%exclude %{_bindir}/yaml-bench
%exclude %{_bindir}/lli-child-target
%exclude %{_bindir}/llvm-isel-fuzzer
%exclude %{_bindir}/llvm-opt-fuzzer
%{_datadir}/opt-viewer
%{_mandir}/man1/*
%exclude %{_mandir}/man1/llvm-config*

%files libs
%license LICENSE.TXT
%{_libdir}/libLLVM-%{maj_ver}.so
%{_libdir}/libLLVM-%{maj_ver}.%{min_ver}*.so
%{_libdir}/LLVMgold.so
%{_libdir}/bfd-plugins/LLVMgold.so
%{_libdir}/libLTO.so*
%{_libdir}/libRemarks.so*

%files devel
%license LICENSE.TXT
%{_bindir}/llvm-config
%{_bindir}/llvm-config-%{maj_ver}
%{_libdir}/libLLVM.so
%{_libdir}/cmake/llvm
%{_libdir}/*.a
%exclude %{_libdir}/libLLVMTestingSupport.a
%{_includedir}/llvm
%{_includedir}/llvm-c
%{_mandir}/man1/llvm-config*

%files cmake-utils
%license LICENSE.TXT
%{_datadir}/llvm/cmake

%files test
%license LICENSE.TXT
%{_bindir}/not
%{_bindir}/count
%{_bindir}/yaml-bench
%{_bindir}/lli-child-target
%{_bindir}/llvm-isel-fuzzer
%{_bindir}/llvm-opt-fuzzer
%{_libdir}/libLLVMTestingSupport.a
%{_libdir}/libLLVMTestingAnnotations.a
%{_libdir}/libllvm_gtest.a
%{_libdir}/libllvm_gtest_main.a
%{_includedir}/llvm-gtest
%{_includedir}/llvm-gmock

%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 16.0.6-2
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Jun 30 2023 kianli <kianli@tencent.com> - 16.0.6-1
- Upgrade to 16.0.6

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 14.0.5-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 14.0.5-3
- Rebuilt for OpenCloudOS Stream 23

* Thu Jan 05 2023 rockerzhu <rockerzhu@tencent.com> - 14.0.5-2
- Disable INSTANTIATE_TEST_SUITE_P on aarch64 due to failing in compiling.

* Wed Jun 15 2022 metrechen <metrechen@tencent.com> - 14.0.5-1
- Release tllvm, based on LLVM 14.0.5

* Thu Jun 02 2022 rockerzhu <rockerzhu@tencent.com> - 14.0.0-1
- Initial build
